package LRRD;
#!/usr/bin/perl
# Nagios


use Exporter;
@ISA = ('Exporter');
@EXPORT = ('lrrd_trend', 
	   'lrrd_fetch', 
	   'lrrd_nscasend', 
	   'lrrd_createlock',
	   'lrrd_removelock',
	   'lrrd_runlock',
	   'lrrd_readconfig',
	   'lrrd_writeconfig',
	   'lrrd_delete',
	   'lrrd_overwrite',
	   'lrrd_config',
	   'lrrd_get_filename');

use strict;
use Config::General;
use RRDs;
use Fcntl qw(:DEFAULT :flock);
use IO::Handle;

my $nsca = new IO::Handle;
my $config = undef;

my $DEBUG=0;
my $configfile="/etc/lrrd/server.conf";

sub lrrd_trend {
    my (@array) = @_;
    return ($array[$#array] - $array[0]);
}

sub lrrd_fetch {
    my ($file,$last,$type) = @_;
    my ($start,$step,$names,$data) = RRDs::fetch $file,$type || "AVERAGE";
    my @array = map { @$_[0] } splice(@$data, $#$data - ($last || 1));
    return $array[0] if (!$last);
    return @array;
}

sub lrrd_nscasend {
    my ($name,$service,$label,$level,$comment) = @_;

    if (!$nsca->opened)
    {
	open ($nsca ,"|$config->{nsca} $config->{nsca_server} -c $config->{nsca_config} -to 60");
    }
    print $nsca  "$name\t$service: $label\t$level\t$comment\n";
}

sub lrrd_createlock {
  my ($lockname) = @_;
  if (-e $lockname && (! -w $lockname || ! -f $lockname)) {
      die "Error writing to $lockname, wrong permissions";
  }
  if (sysopen (LOCK,$lockname,O_WRONLY | O_CREAT | O_EXCL)) {
    print "Creating lock : $lockname succeded\n" if $DEBUG;
    print LOCK $$; # we want the pid inside for later use
    close LOCK;
    return 1;
  } else {
    print "Creating lock : $lockname failed, skipping\n" if $DEBUG;
    return 0;
  }
}

sub lrrd_removelock {
  my ($lockname) = @_;
  if (-e $lockname && ! -w $lockname) {
      die "Error deleting $lockname, wrong permissions";
  }
  if (-e $lockname) {
    unlink $lockname;
    print "Deleting lock : $lockname\n" if $DEBUG;
  } else {
    print "Deleting lock : $lockname not found, skipping\n" if $DEBUG;
  }
}

sub lrrd_runlock {
  my ($lockname) = @_;
    unless (&lrrd_createlock($lockname)) {
      # Is the lockpid alive?
      open LOCK,$lockname;
      my $pid = <LOCK>;
      if (kill(0,$pid)) {
	print "Lock already exists: $lockname. Dying.\n";
	exit 0;
      }
      &lrrd_removelock ($lockname);
      &lrrd_createlock($lockname);
    }
}


sub lrrd_delete {
    my ($config,$data) = @_;
    for my $domain (keys %{$data->{domain}}) {
	unless ($config->{domain}->{$domain}) {
	    logger("Removing domain: $domain");
	    delete ($data->{domain}->{$domain});
	    next;
	}
	for my $node (keys %{$data->{domain}->{$domain}->{node}}) {
	    unless ($config->{domain}->{$domain}->{node}->{$node}) {
		logger("Removing node from $domain: $node");
		delete ($data->{domain}->{$domain}->{node}->{$node});
	    }
	}
    }
    return ($data);
}
sub lrrd_overwrite {
    my ($configfile,$overwrite) = @_;
    for my $key (keys %$overwrite) {
	if (ref $overwrite->{$key}) {
	    &lrrd_overwrite($overwrite->{$key},$configfile->{$key});
	}
	$configfile->{$key} = $overwrite->{$key};
    }
    return ($configfile);
}

sub lrrd_readconfig {
    my ($conf, $missingok) = @_;
    $conf ||= $configfile;
    if (! -r $conf and ! $missingok) {
		print "lrrd_readconfig: cannot open '$conf'\n";
		return undef;
    }
    my $conffile = new Config::General($conf);
    my $config = { $conffile->getall };
    return ($config);
}

sub lrrd_writeconfig {
    my ($datafilename,$data) = @_;
    my $datafile = new Config::General();
    $datafile->save_file($datafilename,$data);
}
    
sub lrrd_config {
    my $conffile = shift;
    $conffile ||= $configfile;
    $config = &lrrd_readconfig ($conffile);
    my $data = &lrrd_readconfig("$config->{dbdir}/datafile", 1);
    
    $data = &lrrd_overwrite($data,$config);
    return ($data);
}

sub lrrd_get_filename {
	my ($domain,$node,$service,$field) = @_;

	return ($config->{'dbdir'} . "/$domain/$node-$service-$field-" . lc substr (($config->{domain}->{$domain}->{node}->{$node}->{client}->{$service}->{$field.".type"}||"GAUGE"), 0,1). ".rrd");

}
